// ************************* Geodetic Computer **************************
// Support routines
// **********************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "geofile.h"

LOCATION *place = NULL;             // array of remote LOCATIONs

char fCountries[160] = "";

// Load the data file containing the LOCATION members into the
// array 'places'

int numplaces = 0;
int iSort = 0;               // sorted on prefix = 0, name = 1

void WriteGeoFile (char *fname)
{
  FILE *countries;
  int  n = 0;

  if (fname != NULL)
    strncpy (fCountries,fname,159);

  if (strlen (fCountries) == 0)
    return;
    
  countries = fopen( fCountries, "w" );

  if ( countries == NULL )
    return;

  for( n = 0; n < numplaces; n++ )
    fprintf( countries, "%6s|%24s|%2s %2s %s|%3s %2s %s|\n",
            place[n].prefix, place[n].name,
            place[n].latDeg, place[n].latMin, place[n].latNS, 
            place[n].lonDeg, place[n].lonMin, place[n].lonEW );

  fclose( countries);
}

int ReadGeoFile (char *fname)
{
  FILE *countries;
  int  n = 0;
  char line[100];
  char sScan[] = "%[^|]%*c%[^|]%*c%[^|]%*c%[^|]%*c\n";
  char lat[12], lon[12];

  if (fname == NULL)
    strcpy (fCountries,"./country.pos");
  else
    strncpy (fCountries,fname,159);
    
  countries = fopen( fCountries, "r" );

  if ( countries == NULL )
    return -1;

  numplaces = 0;
  iSort = 0;
  while( !feof( countries ) ) {
    fscanf( countries, "%[^\n]%*c", line );
    numplaces++;
  }
  
  
  numplaces--;
  rewind(countries);

  if (place != NULL)
    free (place);
    
  place = ( LOCATION *) malloc( numplaces * sizeof( LOCATION ) );
  
  for( n = 0; n < numplaces; n++ ) {
    fscanf( countries, sScan,
            place[n].prefix, place[n].name, lat, lon );
    sscanf( lat, "%s%s%s",
            place[n].latDeg, place[n].latMin, place[n].latNS );
    sscanf( lon, "%s%s%s",
            place[n].lonDeg, place[n].lonMin, place[n].lonEW );
  }
  fclose( countries);
  return 0;
}

int addPlace (
      char *prefix, char *name,
      char *latDeg, char *latMin, char *latNS,
      char *lonDeg, char *lonMin, char *lonEW )
{
  LOCATION *place2 = (LOCATION *) malloc( (numplaces+1)*sizeof(LOCATION));
  if (place2 == NULL) return 0;
  memcpy (place2, place, numplaces * sizeof(LOCATION));
  free (place);
  place = place2;
  strcpy (place[numplaces].prefix, prefix);
  strcpy (place[numplaces].name, name);
  strcpy (place[numplaces].latDeg, latDeg);
  strcpy (place[numplaces].latMin, latMin);
  strcpy (place[numplaces].latNS, latNS);
  strcpy (place[numplaces].lonDeg, lonDeg);
  strcpy (place[numplaces].lonMin, lonMin);
  strcpy (place[numplaces].lonEW, lonEW);
  numplaces++;
  return 1;
}

// comparison functions used in qsort for sorting the
// array of places by prefix or by name

int  ComparePrefix( const void *L1, const void *L2 )
{
   LOCATION *Loc1 = (LOCATION *) L1,
            *Loc2 = (LOCATION *) L2;
   return( strcmp( Loc1->prefix, Loc2->prefix ) );
}

int  CompareName( const void *L1, const void *L2 )
{
   LOCATION *Loc1 = (LOCATION *) L1,
            *Loc2 = (LOCATION *) L2;
   return( strcmp( Loc1->name, Loc2->name ) );
}

int  Find( char *prefix, char *name)
{
	for (int i = 1; i < numplaces; i++)
		if (strcmp(prefix, place[i].prefix) == 0 &&
		    strcmp(name, place[i].name) == 0)
		    return i;
	return -1;
}

void SortByPrefix()
{
  qsort( &place[1], numplaces-1, sizeof( LOCATION ), ComparePrefix );
  iSort = 0;
}

void SortByLocation()
{
  qsort( &place[1], numplaces-1, sizeof( LOCATION ), CompareName );
  iSort = 1;
}

void SortLocations()
{
  if( iSort == 0 )
    SortByLocation();
  else
    SortByPrefix();
}

// Convert string lat or lon to decimal equivalent

double value( char *dd, char *mm, char *quad )
{
  double deg;
  deg = 1.0 * atoi(dd) + atoi(mm)/60.0;
  if( *quad == 'W' || *quad == 'w' ||
      *quad == 'S' || *quad == 's' )
    deg = -1.0 * deg;
  return deg;
}

double placeLat(int ndx)
{
	return value(	place[ndx].latDeg,
					place[ndx].latMin,
					place[ndx].latNS );
}

double placeLon(int ndx)
{
	return value(	place[ndx].lonDeg,
					place[ndx].lonMin,
					place[ndx].lonEW );
}

