//----------------------------------------------------------------
// Geodetic Computer main
//----------------------------------------------------------------

#include "geoid.h"
#include "geodialog.h"
#include "geodetic.h"
#include "geofile.h"
#include "maidenhead.h"
#include "version.h"

#include <string>
#include "FL/fl_ask.H"
#include <FL/filename.H>

Fl_Window	*mainWindow;
Maidenhead	myQTH;
Maidenhead	remoteLoc;
geodetic	geo;
char        szHomedir[120];
string		strFname;
int lastselection = -1;

int main(int argc, char **argv) {
	fl_filename_expand(szHomedir, 119, "$HOME");

	strFname = szHomedir;
	strFname.append("/.geoid/country.pos");
  
 	if (ReadGeoFile((char *)strFname.c_str()) == -1) {
 		strFname = szHomedir;
 		strFname.append("/country.pos");
 		if (ReadGeoFile((char *)strFname.c_str()) == -1) {
	  		fl_message("Cannot open %s", (char *)strFname.c_str());
  			exit(0);
 		}
	}

 	mainWindow = geoDialog();
 	if (numplaces > 1) 
 		loadGeoBrowser();
  
 	mainWindow->show();
 	return Fl::run();
}

void About()
{
	fl_message("Geodetic computer\nVersion: "VERSION"\nDave Freese\nw1hkj@@w1hkj.com");
}

void cbSaveQTH()
{
	double val;

	val = atof(txtMyLatDeg->value());
	sprintf(place[0].latDeg,"%2.0f",val);
	val = atof(txtMyLatMin->value());
	sprintf(place[0].latMin,"%2.0f",val);
	if (btnMyNS->value() == 1)
		place[0].latNS[0] = 'S';
	else
		place[0].latNS[0] = 'N';

	val = atof(txtMyLonDeg->value());
	sprintf(place[0].lonDeg,"%3.0f",val);
	val = atof(txtMyLonMin->value());
	sprintf(place[0].lonMin,"%2.0f",val);
	if (btnMyEW->value() == 1)
		place[0].lonEW[0] = 'W';
	else
		place[0].lonEW[0] = 'E';
	WriteGeoFile(NULL);
}

double myLat()
{
	double lat;
	lat = atof(txtMyLatDeg->value()) + atof(txtMyLatMin->value()) / 60.0;
	if (btnMyNS->value() == 1) lat *= -1;
	return lat;
}

double myLon()
{
	double lon;
	lon = atof(txtMyLonDeg->value()) + atof(txtMyLonMin->value()) / 60.0;
	if (btnMyEW->value() == 1) lon *= -1;
	return lon;
}

double remLat()
{
	double lat;
	lat = atof(txtRemLatDeg->value()) + atof(txtRemLatMin->value()) / 60.0;
	if (btnRemNS->value() == 1) lat *= -1;
	return lat;
}

double remLon()
{
	double lon;
	lon = atof(txtRemLonDeg->value()) + atof(txtRemLonMin->value()) / 60.0;
	if (btnRemEW->value() == 1) lon *= -1;
	return lon;
}

void QthLLtoMH()
{
	myQTH.LatLon(myLat(), myLon());
	myQTH.LL_to_MH();
	txtMyGrid->value(myQTH.MH().c_str());
}

void QthMHtoLL()
{
	char numeric[20];
	myQTH.MH(txtMyGrid->value());
	myQTH.MH_to_LL();
	sprintf(numeric,"%3.0f", myQTH.LatDeg());
	txtMyLatDeg->value(numeric);
	sprintf(numeric,"%4.1f", myQTH.LatMin());
	txtMyLatMin->value(numeric);
	if (myQTH.South() == true) {
		btnMyNS->value(1);
		btnMyNS->label("S");
	} else {
		btnMyNS->value(0);
		btnMyNS->label("N");
	}
	sprintf(numeric,"%3.0f", myQTH.LonDeg());
	txtMyLonDeg->value(numeric);
	sprintf(numeric,"%4.1f", myQTH.LonMin());
	txtMyLonMin->value(numeric);
	if (myQTH.West() == true) {
		btnMyEW->value(1);
		btnMyEW->label("W");
	} else {
		btnMyEW->value(0);
		btnMyEW->label("E");
	}
}

void ComputeGrid()
{
	char numeric[20];
	geo.P1(	myLat(), myLon() );
	geo.P2(	remLat(), remLon() );
	geo.compute();
	sprintf(numeric, "%7.0f", geo.Distance());
	txtGridDistance->value( numeric );
	sprintf(numeric, "%7.0f", geo.Bearing());
	txtGridBearing->value( numeric );
}

void remoteLLtoMH()
{
	remoteLoc.LatLon(remLat(), remLon());
	remoteLoc.LL_to_MH();
	txtRemGrid->value(remoteLoc.MH().c_str());
}

void remoteMHtoLL()
{
	char numeric[20];
	remoteLoc.MH(txtRemGrid->value());
	remoteLoc.MH_to_LL();
	sprintf(numeric,"%3.0f", remoteLoc.LatDeg());
	txtRemLatDeg->value(numeric);
	sprintf(numeric,"%4.1f", remoteLoc.LatMin());
	txtRemLatMin->value(numeric);
	if (remoteLoc.South() == true) {
		btnRemNS->value(1);
		btnRemNS->label("S");
	} else {
		btnRemNS->value(0);
		btnRemNS->label("N");
	}
	sprintf(numeric,"%3.0f", remoteLoc.LonDeg());
	txtRemLonDeg->value(numeric);
	sprintf(numeric,"%4.1f", remoteLoc.LonMin());
	txtRemLonMin->value(numeric);
	if (remoteLoc.West() == true) {
		btnRemEW->value(1);
		btnRemEW->label("W");
	} else {
		btnRemEW->value(0);
		btnRemEW->label("E");
	}
}

void loadGeoBrowser()
{
	char line[80];
	selEndPoint->clear();
	for (int n = 1; n < numplaces; n++) {
		sprintf(line, "%6s|%24s|%2s %2s %s|%3s %2s %s\n",
            place[n].prefix, place[n].name,
            place[n].latDeg, place[n].latMin, place[n].latNS, 
            place[n].lonDeg, place[n].lonMin, place[n].lonEW );
		selEndPoint->add(line);
	}
	txtMyLatDeg->value(place[0].latDeg);
	txtMyLatMin->value(place[0].latMin);
	if (place[0].latNS[0] == 'S') {
		btnMyNS->value(1);
		btnMyNS->label("S");
	} else {
		btnMyNS->value(0);
		btnMyNS->label("N");
	}
	if (place[0].lonEW[0] == 'W') {
		btnMyEW->value(1);
		btnMyEW->label("W");
	} else {
		btnMyEW->value(0);
		btnMyEW->label("E");
	}
	txtMyLonDeg->value(place[0].lonDeg);
	txtMyLonMin->value(place[0].lonMin);
	QthLLtoMH();	
}

void reloadGeoBrowser()
{
	char line[80];
	selEndPoint->clear();
	for (int n = 1; n < numplaces; n++) {
		sprintf(line, "%6s|%24s|%2s %2s %s|%3s %2s %s\n",
            place[n].prefix, place[n].name,
            place[n].latDeg, place[n].latMin, place[n].latNS, 
            place[n].lonDeg, place[n].lonMin, place[n].lonEW );
		selEndPoint->add(line);
	}
}

void cbSortLocations()
{
	char prefix[10], name[60];
	if (lastselection >= 0) {
		strcpy(prefix, place[lastselection].prefix);
		strcpy(name, place[lastselection].name);
	}
	if (bSort->value() == 1) {
		SortByLocation();
		bSort->label("Local");
		reloadGeoBrowser();
	} else {
		SortByPrefix();
		bSort->label("Prefix");
		reloadGeoBrowser();
	}
	txtLocDistance->value("");
	txtLocBearing->value("");
	if (lastselection >= 0) {
		int sel = Find(prefix,name);
		if (sel >= 0) {
			selEndPoint->value(sel);
			SelEndPoint(sel);
		}
	}
}

void ComputeEndPoint()
{
	char numeric[20];
	geo.P1( myLat(), myLon() );
	geo.P2( placeLat(lastselection), placeLon(lastselection) );
	geo.compute();
	sprintf(numeric, "%7.0f", geo.Distance());
	txtLocDistance->value( numeric );
	sprintf(numeric, "%7.0f", geo.Bearing());
	txtLocBearing->value( numeric );
}

void SelEndPoint(int nbr)
{
	if (!nbr) return;
	lastselection = nbr;
	ComputeEndPoint();
}
