#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#include "maidenhead.h"

Maidenhead::Maidenhead()
{
	lat = lon = 0;
	latDeg = latMin = 0;
	lonDeg = lonMin = 0;
	south = false;
	west = false;
	strMH = "AA00aa";
}

void	Maidenhead::makeLon()
{
	lon = lonDeg + lonMin / 60.0;
	if (west) 
		lon = 180 - lon;
	else
		lon = 180 + lon;
}

void	Maidenhead::makeLat()
{
	lat = latDeg + latMin / 60.0;
	if (south) 
		lat = 90 - lat;
	else
		lat = 90 + lat;
}

void	Maidenhead::Lon(double deg, double min)
{ 
	if (deg < 0) west = true;
	deg = fabs(deg);
	lonDeg = floor(deg);
	lonMin = min + (deg - lonDeg)*60.0;
	makeLon();
}

void	Maidenhead::Lat(double deg, double min)
{ 
	if (deg < 0) south = true;
	deg = fabs(deg);
	latDeg = floor(deg);
	latMin = min + (deg - latDeg)*60.0;
	makeLat();
}

void	Maidenhead::LatLon(double degLat, double degLon)
{
	Lat(degLat);
	Lon(degLon);
}

void	Maidenhead::LatLon2(double degLat, double minLat, double degLon, double minLon)
{
	Lat(degLat,minLat);
	Lon(degLon,minLon);
}

void	Maidenhead::LL_to_MH()
{
	strMH = "AA00aa";
// Longitude
	strMH[0] += int(lon / 20);
	strMH[2] += (int(lon) % 20) / 2;
//	strMH[4] += int(round( (lon/2 - int(lon/2)) * 24));
	strMH[4] += int(floor( (lon/2 - int(lon/2)) * 24));
//Latitude
	strMH[1] += int(lat / 10);
	strMH[3] += int(lat) % 10;
//	strMH[5] += int(round( (lat - int(lat))*24));
	strMH[5] += int(floor( (lat - int(lat))*24));
}

void	Maidenhead::MH_to_LL()
{
	double ln, lt;
// longitude	
	lon =	20 * (toupper(strMH[0]) - 'A') + 
			2 * (strMH[2] - '0') +
			(toupper(strMH[4]) - 'A' + 0.5) * 5 / 60.0;
	if (lon < 180)
		west = true;
	else
		west = false;
	ln = lon - 180;
	lonDeg = floor(fabs(ln));
	lonMin = (fabs(ln) - lonDeg) * 60.0;
// latitude
	lat = 	10 * (toupper(strMH[1]) - 'A') + 
			(strMH[3] - '0') +
			(toupper(strMH[5]) - 'A' + 0.5) * 2.5 / 60.0;
	if (lat < 90)
		south = true;
	else
		south = false;
	lt = lat - 90;
	latDeg = floor(fabs(lt));
	latMin = (fabs(lt) - latDeg) * 60.0;
}

